<?php

namespace Drupal\trustpilot_api;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;

/**
 * Interface EndpointPluginInterface.
 *
 * @package Drupal\trustpilot_api
 */
interface EndpointPluginInterface extends PluginInspectionInterface, ContainerFactoryPluginInterface {

  /**
   * Human readable plugin name.
   *
   * @return string
   *   Human readable plugin name.
   */
  public function getPluginName();

  /**
   * Get the prepared request path.
   *
   * @return string
   *   Endpoint request path.
   */
  public function getRequestPath(): string;

  /**
   * Get the method for the endpoint request.
   *
   * @return string
   *   HTTP request method like: GET, POST, etc.
   */
  public function getRequestMethod(): string;

  /**
   * Get array of required request parameter keys.
   *
   * @return array
   *   Array of required request parameter keys.
   */
  public function getRequiredParams(): array;

  /**
   * Get array of default request parameters.
   *
   * @return array
   *   Default endpoint request parameters.
   */
  public function getDefaultRequestParams(): array;

  /**
   * Get populated request parameters.
   *
   * @return array
   *   Request specific parameters.
   */
  public function getRequestParams(): array;

  /**
   * Populate the request parameters.
   *
   * @param array $params
   *   Request specific parameters.
   */
  public function setRequestParams(array $params = []);

  /**
   * Whether or not the endpoint can perform the request in its current state.
   *
   * @return bool
   *   True if the request is ready to be performed.
   */
  public function canPerformRequest(): bool;

}
