<?php

namespace Drupal\trustpilot_api;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Http\ClientFactory;

/**
 * Class TrustpilotClient.
 *
 * @package Drupal\trustpilot_api\Service
 */
class TrustpilotApiClient implements TrustpilotApiClientInterface {

  /**
   * Module config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * API client instance.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * Endpoint plugin manager.
   *
   * @var \Drupal\trustpilot_api\EndpointPluginManager
   */
  protected $endpointPluginManager;

  /**
   * Json service.
   *
   * @var \Drupal\Component\Serialization\Json
   */
  protected $json;

  /**
   * Http client options.
   *
   * @var array
   */
  protected $clientOptions = [];

  /**
   * API version number.
   *
   * @var string
   */
  protected $version = 'v1';

  /**
   * Client constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   Drupal core config factory.
   * @param \Drupal\Core\Http\ClientFactory $http_client_factory
   *   Drupal Guzzle HTTP Client factory.
   * @param \Drupal\trustpilot_api\EndpointPluginManager $endpoint_plugin_manager
   *   Endpoint plugin manager.
   * @param \Drupal\Component\Serialization\Json $json
   *   Json service.
   */
  public function __construct(ConfigFactory $config_factory, ClientFactory $http_client_factory, EndpointPluginManager $endpoint_plugin_manager, Json $json) {
    $this->config = $config_factory->get('trustpilot_api.settings');
    $this->clientOptions = [
      'base_uri' => "https://api.trustpilot.com/{$this->version}/",
      'headers' => [
        'apiKey' => $this->config->get('api_key'),
      ],
    ];
    $this->httpClient = $http_client_factory->fromOptions($this->clientOptions);
    $this->endpointPluginManager = $endpoint_plugin_manager;
    $this->json = $json;
  }

  /**
   * {@inheritdoc}
   */
  public function getHttpClient() {
    return $this->httpClient;
  }

  /**
   * {@inheritdoc}
   */
  public function getEndpointPluginManager() {
    return $this->endpointPluginManager;
  }

  /**
   * {@inheritdoc}
   */
  public function request(EndpointPluginInterface $endpoint, array $params = []) {
    $endpoint->setRequestParams($params);
    $response = $this->getHttpClient()->request($endpoint->getRequestMethod(), $endpoint->getRequestPath(), [
      'http_errors' => FALSE,
      'query' => $endpoint->getRequestParams(),
    ]);
    if ($response->getStatusCode() === 200) {
      return $this->json::decode($response->getBody());
    }

    return [];
  }

}
